#!/usr/bin/perl
#notify_by_email
#Description:
#       Send OpMon notification Messages
#       
#Author:
#       Fernando Rocha (fernando.rocha@opservices.com.br)
#Updates:
#       Estevan Ramalho (estevan.ramalho@opservices.com.br)
#Version:
#       1.0 01-10-2008: Initial release
#       1.1 10-10-2008: Added just file option
#       2.0 30-03-2009: Changed plugin header
#                       Added MIME::Lite module to send the e-mails
#                       Added email format option
#                       Added support to OpMon environmental variables
#       2.1 02-04-2009: Changed the html body
#       2.2 03-04-2009: Added Return-Path tag to the header message
#       2.3 09-02-2012: Added ServiceGroup information on notification of services
#       2.4 14-02-2012: Added HostGroup information on notification of hosts
#       2.5 09-11-2012: Added HostAlias information on notification of hosts

BEGIN { 
	push @INC,"/usr/local/opmon/libexec";
}

# Perl Modules
use strict;
use Getopt::Long;
use POSIX;
use File::Basename;
use MIME::Lite;
use opservices;
use Encode;
use Sys::Hostname;

# Setting environment
$ENV{"USER"}="opuser";
$ENV{"HOME"}="/home/opuser";

# globlal variables
our $version = "2.4";
our $name = $NAME;
our $path = $PATH;
our $logo_path = "";
our ($opt_version, $opt_help, $opt_debug , $opt_file, $opt_jfile, $opt_type, $opt_env, $opt_rpath);
our ($opt_Oaddr, $opt_from, $opt_to, $opt_date, $opt_Ntype);
our ($opt_Hname, $opt_Halias, $opt_Haddr, $opt_Hstate, $opt_Houtput, $opt_Hackauthor, $opt_Hackcomm);
our ($opt_Sname, $opt_Sstate, $opt_Soutput, $opt_Sackauthor, $opt_Sackcomm, $opt_Sgroupname,$opt_Hgroupname);

our $usage = '
Usage: '. $0 .' [OPTION]...
Send a OpMon notify by email, for hosts and services.

General Options:
        -e, --env               Get data from environmental variables (service|host)
        -f, --format            Email format (html|txt)
        -O, --Oaddr             OpMon local adress or hostname that will be used on the links
        -r, --rpath             E-mail address to the Return-Path
        -F, --from              E-mail address of the sender
        -T, --to                E-mail address of the recipient ($CONTACTEMAIL$)
        -N, --ntype             OpMon notification type ($NOTIFICATIONTYPE$)
        -D, --date              OpMon date ($SHORTDATETIME$)
            --file              Notification file to append on the message
            --justfile          Just send the file content on the message

Host Options:
        -H, --Hname             Host name ($HOSTNAME$)
	    --Halias		Host alias ($HOSTALIAS$)
            --Haddr             Host adress ($HOSTADDRESS$)
            --Hstate            Host state ($HOSTSTATE$)
            --Houtput           Host output ($HOSTOUTPUT$)
            --Hackauthor        Host acknowledgment author ($HOSTACKAUTHOR$)
            --Hackcomm          Host acknowledgment comment ($HOSTACKCOMMENT$)
Ex.:
./notify_by_email.pl -O 127.0.0.1 -F opmon@opservices.com.br -T fernando.rocha@opservices.com.br -N RECOVERY -D "01/01/1970 00:00:00" -H HOSTNAME --Halias "HOST ALIAS" --Haddr HOSTADDRESS --Hstate UP --Houtput HOSTOUTPUT --Hackauthor HOSTACKAUTHOR --Hackcomm HOSTACKCOMMENT -f html

Service Options:
        -H, --Hname             Host name ($HOSTNAME$)
            --Haddr             Host adress ($HOSTADDRESS$)
            --Hgroupnames       Host groups ($HOSTGROUPNAMES$)
        -S, --Sname             Service name ($SERVICEDESC$)
            --Sstate            Service state ($SERVICESTATE$)
            --Soutput           Service output ($SERVICEOUTPUT$)
            --Sackauthor        Service acknowledgment author ($SERVICEACKAUTHOR$)
            --Sackcomm          Service acknowledgment comment ($SERVICEACKCOMMENT$)
            --Sgroupnames        Service group name ($SERVICEGROUPNAME$)

Ex.:
./notify_by_email.pl -O 127.0.0.1 -F opmon@opservices.com.br -T fernando.rocha@opservices.com.br -N RECOVERY -D "01/01/1970 00:00:00" -H HOSTNAME --Haddr HOSTADDRESS -S SERVICEDESC --Sstate OK --Soutput SERVICEOUTPUT --Sackauthor SERVICEACKAUTHOR --Sackcomm SERVICEACKCOMMENT --file ./arquivo_de_teste.txt --justfile -f txt

Extra Options:

        -h, --help              Display this help and exit
        -v, --version           Output version information and exit

        -d, --debug     1 = Debug
                        2 = Generate log file

';

sub main {
        # Get options
        getOptions();

        logger("Setting subject and mail body...");

        my $subject = get_subject();

        logger($opt_debug,"Subject: $subject");

        my $mailbody = get_body();

        logger($opt_debug,"Mailbody: $mailbody");

        send_email($subject, $mailbody);

}
#--------------------------------------------------------------------------
sub get_admin_email {
        my $handler;
        my @q_params;
        my @rows;
        my $r;
        
        $handler = do_query(
            "select admin_email from nagios_main limit 1",
            \@q_params,
            "opcfg"
        );
	
        # ignore errors and return an invalid address
        if (!defined($handler)) { 
                return "user\@domain.com";
        }
	
        handler_to_array($handler, \@rows);

        # ignore errors and return an invalid address
        if (@rows < 1) {
                return "user\@domain.com";		
        }
	
        $r = pop(@rows);
        return $$r{'admin_email'};
}

#--------------------------------------------------------------------------

sub getOptions {  #command line options

        if($#ARGV < 0){
                usage($usage);
        }

        Getopt::Long::Configure('bundling');
        GetOptions(
                'v|version'             => \$opt_version,
                'h|help'                => \$opt_help,
                'd|debug=i'             => \$opt_debug,
                'e|env=s'               => \$opt_env,
                'f|format=s'            => \$opt_type,
                'r|rpath=s'             => \$opt_rpath,
                'O|Oaddr=s'             => \$opt_Oaddr,
                'F|from=s'              => \$opt_from,
                'T|to=s'                => \$opt_to,
                'D|date=s'              => \$opt_date,
                'N|ntype=s'             => \$opt_Ntype,
                'file=s'                => \$opt_file,
                'justfile'              => \$opt_jfile,
                'H|Hname=s'             => \$opt_Hname,
                'Halias=s'              => \$opt_Halias,
                'Haddr=s'               => \$opt_Haddr,
                'Hstate=s'              => \$opt_Hstate,
                'Houtput=s'             => \$opt_Houtput,
                'Hackauthor=s'          => \$opt_Hackauthor,
                'Hackcomm=s'            => \$opt_Hackcomm,
                'Hgroupname=s'          => \$opt_Hgroupname,
                'S|Sname=s'             => \$opt_Sname,
                'Sstate=s'              => \$opt_Sstate,
                'Soutput=s'             => \$opt_Soutput,
                'Sackauthor=s'          => \$opt_Sackauthor,
                'Sackcomm=s'            => \$opt_Sackcomm,
                'Sgroupname=s'          => \$opt_Sgroupname,

        )or do {
                exit();
        };

        if ($opt_version) {
                print_version($version);
        }

        if ($opt_help){
                usage($usage);
        }

        if (!$opt_type or $opt_type !~ /txt|html/){
                # default type is html
                $opt_type = "html";
        }

        if (($opt_env) and ($opt_env !~ /service|host/)){
                usage("Option Missing:\t-e|--env\t(service|host)\n");
        }

        if(!$opt_Oaddr){
                $opt_Oaddr = hostname();
        }

        if(!$opt_from){
                $opt_from = get_admin_email();
        }

        if(!$opt_rpath){
                $opt_rpath = get_admin_email();
        }

        if (!$opt_env){
                if ((!$opt_to) or (!$opt_date) or (!$opt_Ntype) or (!$opt_Hname) or (!$opt_Haddr) or (!$opt_Oaddr)){
                        usage($usage);
                }

                if ((!$opt_Sname) and ((!$opt_Hstate) or (!$opt_Houtput))){
                        usage($usage);
                }

                if ((!$opt_Hstate) and ((!$opt_Sstate) or (!$opt_Soutput))){
                        usage($usage);
                }
        } else {
                get_env();
        }

        if (($opt_jfile) and (!$opt_file)){
                usage("Option Missing:\t--file\n");
        }

        if (($opt_debug) and ($opt_debug !~/1|2/)){
                usage("Option Missing:\t-d|--debug\t(1|2)\n");
        }

}

#--------------------------------------------------------------------------
sub get_subject(){
        my $subject;
        if ($opt_Sstate){
                $subject = "** OpMon Alert - $opt_Ntype - $opt_Hname/$opt_Sname is $opt_Sstate **";
        }else{
                $subject = "OpMon Alert - Host $opt_Hname is $opt_Hstate!";
        }
        return ($subject);
}
#--------------------------------------------------------------------------
sub get_footer(){
        my $footer;

        if($opt_type =~ /html/) {
                if($opt_Sstate){
                        #$footer = '            <p><b>Serviços adicionais:</b><br>Para ver informações sobre o serviço, clique abaixo:<br><a href="https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/extinfo/action/service/host_name/'. $opt_Hname .'/service_description/'. $opt_Sname .'/" target="_blank">https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/extinfo/action/service/host_name/'. $opt_Hname .'/service_description/'. $opt_Sname .'/</a><br><br>Para ver informações sobre todos os serviços deste host, clique abaixo:<br><a href="https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/statusgrid/action/services/?useFilter=1&filterString=host_name;'. $opt_Hname .'" target="_blank">https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/statusgrid/action/services/?useFilter=1&filterString=host_name;'. $opt_Hname .'</a></p>';
                        $footer = '            <p><br><a href="https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/extinfo/action/service/host_name/'. $opt_Hname .'/service_description/'. $opt_Sname .'/" target="_blank">Informa&ccedil;&otilde;es sobre o servi&ccedil;o</a><br><a href="https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/statusgrid/action/services/?useFilter=1&filterString=host_name;'. $opt_Hname .'" target="_blank">Informa&ccedil;&otilde;es sobre todos os servi&ccedil;os deste host</a></p>';
                }else{
                        #$footer = '            <p><b>Serviços adicionais:</b><br>Para ver informações sobre o host, clique abaixo:<br><a href="https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/extinfo/action/host/host_name/'. $opt_Hname .'/" target="_blank">https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/extinfo/action/host/host_name/'. $opt_Hname .'/</a></p>';
                        $footer = '            <p><br><a href="https://'. $opt_Oaddr .'/opmon/seagull/www/index.php/extinfo/action/host/host_name/'. $opt_Hname .'/" target="_blank">Informa&ccedil;&otilde;es sobre o host</a></p>';
                }

                $footer .= "<!--$opt_Hname;$opt_Sname;$opt_Sstate$opt_Hstate-->";


        } else {
                if($opt_Sstate) {
                        $footer = "Para ver informações sobre o serviço, clique abaixo:\n\nhttps://$opt_Oaddr/opmon/seagull/www/index.php/extinfo/action/service/host_name/$opt_Hname/service_description/$opt_Sname/\n\nPara ver informações sobre todos os Serviços deste host, clique abaixo:\n\nhttps://$opt_Oaddr/opmon/seagull/www/index.php/statusgrid/action/services/?useFilter=1&filterString=host_name;$opt_Hname\n";
                } else {
                        $footer = "Para ver informações sobre o host, clique abaixo:\n\nhttps://$opt_Oaddr/opmon/seagull/www/index.php/extinfo/action/host/host_name/$opt_Hname/\n";
                }
        }

        return ($footer);
}
#--------------------------------------------------------------------------
sub get_file(){

        return if(!$opt_file);

        logger($opt_debug, "File: $opt_file");

        open (FILE, $opt_file) 
         or do {
                print "$opt_file: $!\n";
                exit(2);
        };
        my @file_content = <FILE>;
        close(FILE);

        my $fc;
        foreach my $line (@file_content){ 
                $line =~ s/[\n\r]/<br>/g if ($opt_type =~ /html/);
                $fc .= $line;
        }

        $fc = '<p><b>File Info:</b><br>'. $fc .'</p>' if ($opt_type =~ /html/);

        return ($fc);


}
#--------------------------------------------------------------------------
sub get_body(){
        my $body; 

        if ($opt_jfile){

                $body = "
***** OpMon *****
".get_file().
"***** OpMon *****
";

                return ($body);
        }

        if ($opt_type =~ /html/){
                $body = '
<html>
<head>
<title>OpMon Alerta</title>
<meta name="generator" content="Namo WebEditor">
<style>'
. get_style() .
'</style>
</head>
<body leftmargin="0" marginwidth="0" topmargin="0" marginheight="0">

<table border="0" width="540" cellspacing="0" cellpadding="0">
    <tr>
        <td colspan="7"><img src="cid:header.png" height="80" width="560" border="0"></td>

    </tr>

    
    <tr>
        <td rowspan="4"><img src="cid:spacer.png" border="0" width="10" height="10"></td>
        <td bgcolor="#E1E1E1" rowspan="3"><img src="cid:spacer.png" width="1" height="10" border="0"></td>
        <td rowspan="2"><img src="cid:spacer.png" width="30" height="10" border="0"></td>
        <td>

            <p class="titulo1">Alerta OpMon</p>
            <p><font color="#C58400">&bull;</font> &nbsp;Notification Type: <b>'. $opt_Ntype .'</b><br>';

        if ($opt_Hackauthor ne "" || $opt_Sackauthor ne "") {
                $body .= '<font color="#C58400">&bull;</font> &nbsp;Author: <b>'. $opt_Hackauthor.' '.$opt_Sackauthor .'</b><br>';
                $body .= '<font color="#C58400">&bull;</font> &nbsp;Comment: <b>'. decode_utf8($opt_Hackcomm).' '.decode_utf8($opt_Sackcomm) .'</b><br>';
        }

       	$body .= '<font color="#C58400">&bull;</font> &nbsp;Host: <b>'. $opt_Hname .'</b><br>
	     <font color="#C58400">&bull;</font> &nbsp;Host Description: <b>'. $opt_Halias .'</b><br>'
. get_hstate() .
'            <font color="#C58400">&bull;</font> &nbsp;Address: <b>'. $opt_Haddr .'</b><br>'
. get_sstate() . 
'            <font color="#C58400">&bull;</font> &nbsp;Date/Time: <b>'. $opt_date .'</b></p>
            <p><b>Additional Info:</b><br>'. $opt_Houtput . $opt_Soutput .'</p>

'. get_file() . get_footer() .
'            <p class="txt_p">&nbsp;</p>
           
        </td>
        <td rowspan="2"><img src="cid:spacer.png" width="30" height="10" border="0"></td>
        <td bgcolor="#E1E1E1" rowspan="3"><img src="cid:spacer.png" width="1" height="10" border="0"></td>
        <td rowspan="4"><img src="cid:spacer.png" width="10" height="10" border="0"></td>
    </tr>
    <tr>
        <td><img src="cid:spacer.png" width="475" height="1" border="0"></td>
    </tr>
    <tr>

        <td height="70" bgcolor="#F1F1F1">&nbsp;</td>
        <td height="70" bgcolor="#F1F1F1">
            <p class="txt_p"><img src="cid:logo.png" alt="Logo" align="right" border="0" style="margin-left: 20px;"></a>Mensagem gerada automaticamente.<br>Em caso de d&uacute;vida, entre em contato com nosso suporte.</p>

        </td>
        <td height="70" bgcolor="#F1F1F1">&nbsp;</td>

    </tr>
    <tr height="15">
        <td colspan="5" class="footer" align="center" bgcolor="#DF9707"><p class="txt_p">&nbsp;</p></td>
    </tr>

</table>
</body>
</html>
';

        }else{
                $body = "
***** OpMon *****

Notification Type: $opt_Ntype $opt_Hackauthor$opt_Sackauthor
$opt_Hackcomm$opt_Sackcomm

Host: $opt_Hname
Host Description: $opt_Halias
". get_hstate() .
"Address: $opt_Haddr"
. get_sstate() ."

Date/Time: $opt_date

Additional Info:

$opt_Houtput$opt_Soutput

"
. get_file() .
"
***** OpMon *****

Serviços adicionais:

"
. get_footer() .
"";
        }

        return ($body);
}
#--------------------------------------------------------------------------
sub get_header(){
        my $header;

        if($opt_Sstate){
                $header = "service_". $opt_Sstate .".png";
        }else{
                $header = "host_". $opt_Hstate .".png";
        }

        logger($opt_debug,"Header: $header");

        return($header);
}
#--------------------------------------------------------------------------
sub get_hstate(){
        my $hstate;

        if($opt_type =~ /html/){
                if(!$opt_Sstate){
                        $hstate = '             <font color="#C58400">&bull;</font> &nbsp;State: <b>'.$opt_Hstate.'</b><br>
            <font color="#C58400">&bull;</font> &nbsp;Host Group: <b>'.$opt_Hgroupname.'</b><br>';
                }
        }else{
                if(!$opt_Sstate){
                        $hstate = 'Host Groups: '.$opt_Hgroupname.'
State: '.$opt_Hstate.'
';
                }
        }

        return($hstate);
}
#--------------------------------------------------------------------------
sub get_sstate(){

        return if (!$opt_Sstate);

        my $sstate;

        if($opt_type =~ /html/){
                $sstate = '             <font color="#C58400">&bull;</font> &nbsp;Service: <b>'.$opt_Sname.'</b><br>
            <font color="#C58400">&bull;</font> &nbsp;Service Group: <b>'.$opt_Sgroupname.'</b><br>
            <font color="#C58400">&bull;</font> &nbsp;State: <b>'.$opt_Sstate.'</b><br>';
        }else{
                $sstate = '
Service: '.$opt_Sname.'
Service Group: '.$opt_Sgroupname.'
State: '.$opt_Sstate.'';
        }

        return($sstate);
}

#--------------------------------------------------------------------------
sub send_email(){

        my $subject = shift;
        my $mailbody = shift;

        my $header = get_header();
        my $type;
        if($opt_type =~ /html/){
                $type = "multipart/related";
        }else{
                $type = "text";
        }

        my $msg;
        MIME::Lite->send("sendmail", "/usr/lib/sendmail -f $opt_rpath -t -oi");
        if ($opt_type =~ /html/){
                # create a new MIME Lite based email
                $msg = MIME::Lite->new(
                        Subject => $subject,
                        From    => $opt_from,
                        To      => $opt_to,
                        Type    => $type,
                        "Return-Path" => "$opt_rpath", 
                );
                $msg->attach(
                        Type => 'text/html',
                        Data => $mailbody,
                );
        }else{
                # create a new MIME Lite based email
                $msg = MIME::Lite->new(
                        Subject => $subject,
                        From    => $opt_from,
                        To      => $opt_to,
                        Data    => $mailbody,
                        "Return-Path" => "$opt_rpath", 
                );
        }

        if (($opt_type =~ /html/) and (!$opt_jfile)){
                $msg->attach(
                        Type => 'image/png',
                        Id   => 'header.png',
                        Path => "$path/images/$header",
                );
                $msg->attach(
                        Type => 'image/png',
                        Id   => 'spacer.png',
                        Path => "$path/images/spacer.png",
                );
                
                $logo_path = `/usr/local/opmon/utils/opmon-export-logo.php`;
                
                if ($logo_path eq "") {
                	$logo_path = "$path/images/logo.png";
                }
                
                $msg->attach(
                        Type => 'image/png',
                        Id   => 'logo.png',
                        Path => "$logo_path",
                );
        }

        $msg->send();
}
#--------------------------------------------------------------------------
sub get_env(){

        logger($opt_debug,"Loading Environmental Variables");

        $opt_to    = $ENV{'NAGIOS_CONTACTEMAIL'};
        $opt_Ntype = $ENV{'NAGIOS_NOTIFICATIONTYPE'}; 
        $opt_date  = $ENV{'NAGIOS_SHORTDATETIME'}; 
        $opt_Hname = $ENV{'NAGIOS_HOSTNAME'}; 
        $opt_Haddr = $ENV{'NAGIOS_HOSTADDRESS'};
		$opt_Halias = $ENV{'NAGIOS_HOSTALIAS'};

        if ($opt_env =~ /host/){ 
                $opt_Hstate     = $ENV{'NAGIOS_HOSTSTATE'}; 
                $opt_Houtput    = $ENV{'NAGIOS_HOSTOUTPUT'}; 
                $opt_Hackauthor = $ENV{'NAGIOS_HOSTACKAUTHOR'}; 
                $opt_Hackcomm   = $ENV{'NAGIOS_HOSTACKCOMMENT'};
                $opt_Hgroupname = $ENV{'NAGIOS_HOSTGROUPNAMES'}
        }else{ 
                $opt_Sname      = $ENV{'NAGIOS_SERVICEDESC'}; 
                $opt_Sstate     = $ENV{'NAGIOS_SERVICESTATE'}; 
                $opt_Soutput    = $ENV{'NAGIOS_SERVICEOUTPUT'}; 
                $opt_Sackauthor = $ENV{'NAGIOS_SERVICEACKAUTHOR'}; 
                $opt_Sackcomm   = $ENV{'NAGIOS_SERVICEACKCOMMENT'};
                $opt_Sgroupname = $ENV{'NAGIOS_SERVICEGROUPNAMES'}; 
        }

        return;
}
#--------------------------------------------------------------------------
sub get_style(){
        my $style = '
body {
        background: #FFFFFF;
        font-family: Tahoma; 
}

p {
        font-size: 11px; 
        color: #1A1A1A;
}

.titulo1 {
        font-size: 11pt; 
        color: #C58400;
        font-weight:bold;
}

a{
        color: #2559BF;
        text-decoration: none;
}

a:hover{
        text-decoration: underline;
}

.footer td{
        background: #Df9707;
        color: #FFFFFF;
        text-align: center;
        vertical-align: bottom;
        height: 25px;
    font-size: 9px;
}
.footer a{
        text-decoration: none;
        color: #FFFFFF;
}
.footer a:hover{
        text-decoration: underline;
        color: #FFFFFF;
}


.txt_p {
font-size: 9px;
}
';
        return ($style);
}

&main;
