# opservices.pm
#
# Description:
#	Opservices Perl Library
#	This Library provides functions to help the plugin developement
#               
# Version:
# 	1.0 17-03-09: Initial release
#	2.0 19-11-13: Added functions to deal with database	

package opservices;

use Exporter;
use strict;
use Getopt::Long;
use POSIX;
use File::Basename;
use DBI;
use Data::Dumper;

our @ISA = qw(Exporter);

# To make the functions accessible 
# from plugin's context they should 
# be declared here
our @EXPORT = qw(
    $NAME 
    $PATH 
    $TIMEOUT 
    %ERRORS 
    &print_version 
    &support 
    &usage 
    &logger 
    &epoch_to_date 
    &date_to_epoch 
    &unique
    &parse_db_conf
    &get_opmon_dbhost
    &get_opmon_dbport
    &get_opmon_dbuser
    &get_opmon_dbpass
    &get_opmon_dsn
    &do_query
    &handler_to_array
);

our %global_db_conf;
our $NAME = basename($0, ".pl");
our $PATH =  dirname($0);
our $TIMEOUT = 60;
our %ERRORS= (
    'OK'=>0,
    'WARNING'=>1,
    'CRITICAL'=>2,
    'UNKNOWN'=>3,
    'DEPENDENT'=>4
);

# convert a do_query's handler
# return into an array of hashes
sub handler_to_array {
	my $handler;
	my $hash;
	my $ref;

	$handler = shift;
	$a = shift;

	while($ref = $$handler->fetchrow_hashref()) {
		push(@$a, $ref);
	}
}

# run query and return a reference
# to handler
sub do_query {
	my $query;
	my $database;
	my $params;
	my $dsn;
	my $user;
	my $pass;
	my $dbh;
	my $qh;

	$query = shift;
	$params = shift;
	$database = shift;

	$dsn = get_opmon_dsn($database);
	return undef if (!defined $dsn);

	$user = get_opmon_dbuser();
	return undef if (!defined $user);

	$pass = get_opmon_dbpass();
	return undef if (!defined $pass);

	$dbh = DBI->connect($dsn, $user, $pass) or return undef;
	$qh = $dbh->prepare($query);

	$qh->execute(@$params);
	return \$qh;
}

# parse configuration file and
# return complete dsn for provided
# database
sub get_opmon_dsn {
	my $database = shift;
	my $host;
	my $port;

	$host = get_opmon_dbhost();
	return undef if (!defined $host);

	$port = get_opmon_dbport();
	return undef if (!defined $port);

	return "DBI:mysql:database=$database;host=$host;port=$port";
}

# return configured mysql password
# or undefined if error
sub get_opmon_dbpass {
	my %c;
	return undef if (parse_db_conf(\%c) == undef);
	return $c{'dbpass'};
}

# return configured mysql
# database user or undefined
# if error
sub get_opmon_dbuser {
	my %c;
	return undef if (parse_db_conf(\%c) == undef);
	return $c{'dbuser'};
}

# return configured mysql port or
# undefined in case of error
sub get_opmon_dbport {
	my %c;
	return undef if (parse_db_conf(\%c) == undef);
	return $c{'dbport'};
}

# return configured mysql ip address
# or undefined in case of error
sub get_opmon_dbhost {
	my %c;
	return undef if (parse_db_conf(\%c) == undef);
	return $c{'dbhost'};
}

# parse opmon db configuration file
# and returns a hash with 
sub parse_db_conf {
	my $db_config_file = "/usr/local/opmon/etc/db.php";
	my $conf = shift;

	return undef if (! -f $db_config_file);

	if (defined(%global_db_conf)) {
		foreach(keys %global_db_conf) {
			$$conf{$_} = $global_db_conf{$_};
		}
		return 1;
	}

	$global_db_conf{'dbhost'} = "127.0.0.1";
	$global_db_conf{'dbport'} = 3306;
	$global_db_conf{'dbuser'} = "root";
	$global_db_conf{'dbpass'} = "";

	open(DBFILE,$db_config_file) || return undef;
	foreach (<DBFILE>) {
		next if (!m/\s*\$(.{6})\s*=\s*"(.*)".*;$/);
		$global_db_conf{'dbpass'} = $2 if ($1 eq "DBPASS");
		$global_db_conf{'dbuser'} = $2 if ($1 eq "DBUSER");
		$global_db_conf{'dbport'} = $2 if ($1 eq "DBPORT");
		$global_db_conf{'dbhost'} = $2 if ($1 eq "DBHOST");
	}
	close(DBFILE);

	foreach(keys %global_db_conf) {
		$$conf{$_} = $global_db_conf{$_};
	}
	return 1;
}

# print plugin version text
# Ex.:
# print_version("1.0");
sub print_version {
	my $commandName = $NAME;
	my $pluginRevision = shift;
	$pluginRevision =~ s/^\$Revision: //;
	$pluginRevision =~ s/ \$\s*$//;
	print "$commandName $pluginRevision\n";
	print "This plugin come with ABSOLUTELY NO WARRANTY. You may redistribute\ncopies of the plugins under the terms of the GNU General Public License.\nFor more information about these matters, see the file named COPYING.\n";
	exit $ERRORS{'UNKNOWN'};
}

# Show the support message
# Ex.:
# support();
sub support {
	my $support='Send email to suporte@opservices.com.br if you have questions\nregarding use of this software. To submit patches or suggest improvements,\nsend email to suporte@opservices.com.br.\nPlease include version information with all correspondence (when possible,\nuse output from the --version option of the plugin itself).\n';
	$support =~ s/@/\@/g;
	$support =~ s/\\n/\n/g;
	print $support;
}

# Show the usage message
# Ex.:
# usage();
sub usage {
	print @_;
	support();
	exit $ERRORS{'UNKNOWN'};
}

# Check a valid host name
# To use you must specify the following parameter:
#       $host1
# Ex.:
# is_hostname("www.opservices.com.br");
sub is_hostname {
	my $host1 = shift;
	return 0 unless defined $host1;
	
	if ($host1 =~ m/^[\d\.]+$/ && $host1 !~ /\.$/) {
		
		if ($host1 =~ m/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/) {
			return 1;
		}
		
		return 0;
	} 
	
	if ($host1 =~ m/^[a-zA-Z0-9][-a-zA-Z0-9]*(\.[a-zA-Z0-9][-a-zA-Z0-9]*)*\.?$/) {
		return 1;
	}
	
	return 0;
}

# Show or Store debug messages
# To use you must specify the following parameter:
#       $opt	(1-show, 2-log)
#	$msg
#
# The log file will be created at the current $path, called <plugin_name>.log
#
# Ex.:
# logger(1,"www.opservices.com.br");
sub logger {

	my $opt = shift;
	my $msg = shift;
	my $log = "$PATH/$NAME.log";

	my $date = strftime("%d/%m/%Y - %H:%M:%S",localtime);

	if ($opt == 1){
		print "$msg\n";
		return;
	} 
	
	if ($opt == 2) {
		open(LOG, ">>$log");
		print LOG "$date => $msg\n";
		close(LOG);
	}
}

# Convert date format, from Unix timestamp to date
# To use you must specify the following parameter:
#       $epoch
#
# Ex.:
# epoch_to_date("1237322708");
sub epoch_to_date {

	my $epoch = shift;

	my $date = ctime ($epoch);
	chomp($date);

	return ($date);
}

# Convert date format, from date to Unix timestamp
# To use you must specify the following parameter:
#	$day
#	$mon
#	$year
#	$hour
#	$min
#	$sec
#
# Ex.:
# 30/12/2008 01:30:00
# date_to_epoch(30,12,2008,01,30,00);
sub date_to_epoch {

	# 30/12/2008 01:30:00
	# date_to_epoch(30,12,2008,01,30,00);

	my $day = shift;
	my $mon = shift;
	my $year = shift;
	my $hour = shift;
	my $min = shift;
	my $sec = shift;
	my $wday = 0;
	my $yday = 0;

	my $epoch = mktime($sec,$min,$hour,$day,($mon-1),($year-1900),$wday,$yday,-1);

	return($epoch)
}

# Print the message and exits with nagios OK status
# To use you must specify the following parameter:
#       $msg
#
# Ex.:
# print_ok("teste");
sub print_ok {
	my $msg = shift;
	print $msg;
	exit $ERRORS{'OK'};
}

# Print the message and exits with nagios CRITICAL status
# To use you must specify the following parameter:
#       $msg
#
# Ex.:
# print_crit("teste");
sub print_crit {
        my $msg = shift;
        print $msg;
        exit $ERRORS{'CRITICAL'};
}

# Print the message and exits with nagios WARNING status
# To use you must specify the following parameter:
#       $msg
#
# Ex.:
# print_warn("teste");
sub print_warn {
        my $msg = shift;
        print $msg;
        exit $ERRORS{'WARNING'};
}

# return formated performance data
sub perf_data {
	my $metric = shift;
	my $value = shift;
	my $unit = shift;
	my $warn = shift;
	my $crit = shift;

	return "$metric=$value$unit;$warn;$crit;; ";
}

# wget-style. routine by tachyon
# at http://tachyon.perlmonk.org/
#Ex.:
#	my $count=0;
#	my $total_size = 10000;
#
#	while($count <= $total_size){
#		print progress_bar( $count, $total_size, 50, '=' );
#		$count++;
#	}
#	print "\n";
sub progress_bar {
    my ( $got, $total, $width, $char ) = @_;
    $width ||= 50; $char ||= '=';
    my $num_width = length $total;
    sprintf "|%-${width}s| %${num_width}s of %s (%.2f%%)\r",
        $char x (($width-1)*$got/$total). '>',
        $got, $total, 100*$got/+$total;
}

# Discard all but one of identical entry
# To use you must specify the following parameter:
#       @in
#
# Ex.:
# unique(@in);

sub unique {
	my @in = shift;

	undef my %saw;
	my @out = grep(!$saw{$_}++, @in);

	return (@out);
}

1;
