#!/bin/bash

################################################################################
# TICKET: 1096801
# AUTOR: Guilherme Fontans
# CRIADO EM: 24/09/2015
################################################################################


readonly VERSION="1.0"
readonly OK=0
readonly WARNING=1
readonly CRITICAL=2
readonly UNKNOWN=3
readonly TRUE=1
readonly FALSE=0
readonly URL="[www.opservices.com.br/suporte]"
readonly PORT_DEFAULT=80
readonly COUNT_DEFAULT=5
readonly TIMEOUT_DEFAULT=60
readonly WARNING_VALUE_DEFAULT=200
readonly CRITICAL_VALUE_DEFAULT=300

programName=$( basename $0 )
ssl=${FALSE}
warning=""
critical=""
port=""
host=""
count=""
timeout=""
verbose=${FALSE}
min=""
max=""
avg=""

function getGuideLine {

    cat <<EOF
------------------------------------------
Manual de exemplo.
------------------------------------------

Descrição do plugin:
    O plugin medi a latência e taxa de transferência de um servidor web em determinada
porta, podendo passar por parâmentro o número de execuções do ping, no final é retornado
a média da latência de cada ping.

Dependências:
    httping.x86_64

Funcionamento:
    O plugin recebe do usuário o host para ser verificado e executa o comando httping,
usando a media das execuções do ping para medir os thresholds,caso não seja passado
nenhum outro parâmetro, o plugin assume a porta 80 para ser verificadada, 60sec para
timeout, 200ms para warning, 300ms para crítico, 5 execuções do ping.

Exemplos de execução e retorno do comando:

    ./check_httping.sh -H www.google.com
	Tempo de resposta: 27.4ms | avg=27.4;200;300;0; min=26.5 max=29.0

    ./check_httping.sh -H www.google.com -w 100 -c 200
    Tempo de resposta: 177.5ms | avg=177.5ms;100;200;0; min=43.6ms;;;; max=668.7ms;;;;

    ./check_httping.sh -H www.google.com -w 200 -c 300 --ssl
    Tempo de resposta: 142.4ms | avg=142.4ms;200;300;0; min=104.4ms;;;; max=245.6ms;;;;

    ./check_httping.sh -H www.google.com -n 10 -t 30
    Tempo de resposta: 27.7ms | avg=27.7ms;200;300;0; min=26.6ms;;;; max=29.1ms;;;;

EOF
}

function usage(){

    echo -e "
Usage:
    ${programName} -H <host> -p [<port>] [-w <warning>] [-c <critical>]"

}

function version(){
    echo ${programName} ${VERSION} ${URL}
}

function help(){
    local usage=$( usage )
    local version=$( version )
    cat <<EOF

${version}

Developed by:
    OpServices

Author:
    Guilherme Fontans
${usage}

--usage
    Print usage information
-h, --help
    Print detailed help screen
-V, --version
    Print version information
-H, --host
    Host for verification
--ssl
    Use https
-n,--count
    How many times to ping(default:5)
-p, --port
    Port for verification(default:80)
-w, --warning=INTEGER
   MiliSeconds warning value (default:200)
-c, --critical=INTEGER
   MiliSeconds critical value(default:300)
-t,timeout
   Seconds before plugin times out(default:60)
-v, --verbose
  Show details for command-line debugging
EOF


}

################################################################################
# Seta variaveis definidas conforme parâmetros passados
################################################################################

function getOptions(){
    ARG="$@"
    if [ -z $1 ]; then
        usage
        exit ${OK}
    fi
    eval set -- "$@"

    while true ; do
        case "$1" in
            --usage)
                usage
                exit ${OK}
                ;;
            -h|--help)
                help
                exit ${OK}
                ;;
            -p|--port)
                port=${2}
                shift 2
                ;;
            -n|--count)
                count=${2}
                shift 2
                ;;
            -t|--timeout)
                timeout=${2}
                shift 2
                ;;
            --ssl)
                ssl=${TRUE}
                shift 1
                ;;
            -v|--verbose)
                verbose=${TRUE}
                shift 1
                ;;
            -H|--host)
                host=${2}
                shift 2
                ;;
            -w|--warning)
                warning=${2}
                shift 2
                ;;
            -c|--critical)
                critical=${2}
                shift 2
                ;;
            --man)
                getGuideLine
                exit ${OK}
                ;;
            -V|--version)
                version
                exit ${OK}
                ;;
            *)
                shift
                break;;
        esac
    done


    port=${port:=$PORT_DEFAULT}
    count=${count:=$COUNT_DEFAULT}
    timeout=${timeout:=$TIMEOUT_DEFAULT}
    warning=${warning:=$WARNING_VALUE_DEFAULT}
    critical=${critical:=$CRITICAL_VALUE_DEFAULT}
}

function getHttPing(){
    if [ "${ssl}" -eq ${TRUE} ]; then
        httping=$(/usr/bin/httping -l ${host} -c ${count} -t ${timeout}) 2> /dev/null
    else
        httping=$(/usr/bin/httping ${host} -p ${port} -c ${count} -t ${timeout}) 2> /dev/null
    fi

    echo ${httping}
}

function checkThreshold(){
    local value=${1}
    local status=${OK}

    if [ $(echo "${value} > ${warning} && ${value} < ${critical}" | bc) -eq 1 ]; then
        status=${WARNING}
    elif [ $( echo "${value} > ${critical}" | bc) -eq 1 ]; then
        status=${CRITICAL}
    fi

    echo ${status}
}

################################################################################
# Rotina principal do programa
################################################################################

function main(){
    getOptions $( getopt -a -o ?Vhvp:H:n:w:c:t: -l "verbose usage help warning: timeout: host: critical: port: count: man versioni ssl" -n "$0" -- "$@" )
    regex='([0-9]+[\.,\,][0-9]+)\/([0-9]+[\.,\,][0-9]+)\/([0-9]+[\.,\,][0-9]+)'
    local httping=$( getHttPing )
    if [ ${verbose} -eq ${TRUE} ]; then
        echo "DEBUG: host=${host}"
        echo "DEBUG: port=${port}"
        echo "DEBUG: repeat=${count}"
        echo "DEBUG: ssl=${ssl}"
        echo "DEBUG: timeout=${timeout}"
        echo "DEBUG: warn=${warning}"
        echo "DEBUG: crit=${critical}"
        echo "DEBUG: comand=${httping}"
    fi
    if [[ ${httping} =~ ${regex} ]]; then
        min=${BASH_REMATCH[1]/,/.}
        avg=${BASH_REMATCH[2]/,/.}
        max=${BASH_REMATCH[3]/,/.}
    else
        echo "Endereço não encontrado"
        exit ${UNKNOWN}
    fi
    local outputMessage="Tempo de resposta: ${avg}ms"
    local outputPerfData="| avg=${avg}ms;${warning};${critical};0; min=${min}ms;;;; max=${max}ms;;;;"

    echo ${outputMessage} ${outputPerfData}
    exit $(checkThreshold ${avg})
}
main "$@"
#: set smartindent tabstop=4 shiftwidth=4 softtabstop=4 expandtab
